/*
 *  Copyright (c) 1992, 1993 John E. Davis  (davis@amy.tch.harvard.edu)
 *  All Rights Reserved.
 */
#include <stdio.h>
#include <string.h>
#include <setjmp.h>
#ifdef msdos
#include <dir.h>
#endif

#include "file.h"
#include "buffer.h"
#include "display.h"
#include "sysdep.h"
#include "sig.h"
#include "misc.h"
#include "keymap.h"
#include "screen.h"
#include "ledit.h"
#include "slang.h"
#include "search.h"
#include "text.h"

extern jmp_buf Jump_Buffer;

char *get_cwd(void)
{
   static char cwd[255];

   if (*cwd ==  0)
     {
	getcwd(cwd, 254);
#ifndef VMS
	fixup_dir(cwd);
#endif
     }
   return(cwd);
}

#if 0
void load_init_file(void)
{
   FILE *fp;
#ifdef unix
   char *file = ".jedrc";
#else
   char *file = "jed.rc";
#endif
   char initfile[255], *home;

   home = getenv("JED_HOME");
#ifndef VMS
   if ((home != NULL) || (NULL != (home = getenv("HOME"))))
     {
	strcpy(initfile, home);
	fixup_dir(initfile);
     }
   else initfile[0] = 0;
#else
   if (home == NULL) home = "SYS$LOGIN:";
   strcpy(initfile, home);
#endif

   strcat(initfile, file);
   if ((fp = fopen(initfile,"r")) == NULL) return;
   fclose(fp);
   load_file(initfile);
}
#endif 

int main(int argc, char **argv)
{
   char *cmd_hook = "command_line_hook";
   int i,j;

   
   /* Change case Lut.  Do it now so it can be used */
   for (i = 0; i < 256; i++) 
     {
	Chg_UCase_Lut[i] = i;
	Chg_LCase_Lut[i] = i;
     }
   
   for (i = 'A'; i <= 'Z'; i++) 
     {
	j = i + 32;
	Chg_UCase_Lut[j] = i;
	Chg_LCase_Lut[i] = j;
     }
#ifdef msdos
   /* Initialize for DOS code page 437. */
   Chg_UCase_Lut[135] = 128; Chg_LCase_Lut[128] = 135;
   Chg_UCase_Lut[132] = 142; Chg_LCase_Lut[142] = 132;
   Chg_UCase_Lut[134] = 143; Chg_LCase_Lut[143] = 134;
   Chg_UCase_Lut[130] = 144; Chg_LCase_Lut[144] = 130;
   Chg_UCase_Lut[145] = 146; Chg_LCase_Lut[146] = 145;
   Chg_UCase_Lut[148] = 153; Chg_LCase_Lut[153] = 148;
   Chg_UCase_Lut[129] = 154; Chg_LCase_Lut[154] = 129;
   Chg_UCase_Lut[164] = 165; Chg_LCase_Lut[165] = 164;
   /* Note that Slang uses decimal  instead  of Octal.  Octal below, 
      decimal above */
   define_word("0-9a-z\200-\247\340-\353");
#else
   /* ISO Latin */
   for (i = 192; i <= 221; i++) 
     {
	j = i + 32;
	Chg_UCase_Lut[j] = i;
	Chg_LCase_Lut[i] = j;
     }
   Chg_UCase_Lut[215] = 215; Chg_LCase_Lut[215] = 215;
   Chg_UCase_Lut[223] = 223; Chg_LCase_Lut[223] = 223;
   Chg_UCase_Lut[247] = 247; Chg_LCase_Lut[247] = 247;
   Chg_UCase_Lut[255] = 255; Chg_LCase_Lut[255] = 255;

   define_word("0-9a-z\277-\326\330-\336\340-\366\370-\376");
#endif
   
   init_tty();
   init_display(1);

   /* incase something goes wrong before we even get started... */
   if (setjmp(Jump_Buffer) != 0)
     {
	/* hmm.... just exit I guess */
	exit_error("Fatal Error!");
     }

   if (NULL == (CBuf = make_buffer()))
     {
	exit_error("Allocation Failure.");
     }
   CLine = NULL;
   strcpy(CBuf->name, "*scratch*");
   *CBuf->file = 0;
   strcpy(CBuf->dir, get_cwd());

   /* This order here is crucial! */
   init_keymaps();
   init_minibuffer();

   set_file_modes();
   if (CLine == NULL) make_line(25);
   Point = 0;
   window_buffer(CBuf);

   cls();

   load_file("site.sl");

   if (lang_is_defined(cmd_hook))
     {
	for (i = argc - 1; i > 0; i--) lang_push_string(argv[i]);
	lang_push_integer(argc - 1);
	lang_run_hooks(cmd_hook, NULL);
     }
   else if (argc > 1) find_file_in_window(argv[1]);
#if 0
   /* This has now been replaced by hooks */
     {
	/* load init file unless FIRST parameter "-n" then do not */
	if ((argc > 1) && (!strcmp(argv[1], "-n")))
	  {
	     argc--; argv++;
	  }
	else load_init_file();
	
	while(--argc > 0)
	  {
	     argv++;
	     if (!strcmp("-l", *argv))   /* load file */
	       {
		  argc--;
		  argv++;
		  if (!argc)
		    {
		       exit_error("Expecting load-file name.");
		    }
		  load_file(*argv);
	       }
	     else if (!strcmp("-f", *argv))   /* do function */
	       {
		  argc--;
		  argv++;
		  if (!argc) msg_error("-f what??");
		  else interpret(*argv);
		  Lang_Error = 0;
	       }
	     else if (!strcmp("-g", *argv))
	       {
		  argc--; argv++;
		  if (!argc  || (1 != sscanf(*argv, "%d", &l))) msg_error("-g ??");
		  else goto_line(&l);
	       }
	     
	     else if (!strcmp("-s", *argv))
	       {
		  argc--; argv++;
		  if (!argc) msg_error ("-s ??");
		  else search(*argv, 1, 0);
	       }
	     
	     else
	       {
		  file = *argv;
#ifdef msdos
		  while (*file++ != 0) if (*(file - 1) == '/') *(file - 1) = '\\';
		  file = *argv;
#endif
		  switch_to_buffer(make_buffer());
		  file = expand_filename(file);
		  buffer_filename(file, NULL);
		  n = read_file(file);
		  if (n != -1)
		    {
		       sprintf(msgstr,"%d lines read", n);
		       message(msgstr);
		    }
		  else message("New File.");
		  CLine = CBuf->beg;
		  if (CLine == NULL) make_line(25);
		  Point = 0;
		  set_file_modes();
		  window_buffer(CBuf);
	       }
	  }
     }
#endif
   
	
#ifndef VMS
#ifndef msdos
   init_signals();
#endif
#endif

   /* after we have possible loaded key definitions, we can fix up
      the minibuffer map. This way user definitions are used. */

   if (NULL == (Mini_Map = create_keymap("Mini_Map"))) exit_error("Fatal.");
   undefine_key("^M", Mini_Map);
   define_key1("^M", (VOID *) exit_minibuffer, F_INTRINSIC, Mini_Map);
   define_key1("^I", (VOID *) mini_complete, F_INTRINSIC, Mini_Map);
   define_key1(" ", (VOID *) mini_complete, F_INTRINSIC, Mini_Map);
   /*
   Window->trashed = 1;
   update((Line *) (NULL)); */

   /* edit_loop */
   jed();  /* never returns */
   return(0);  /* compiler does not know it though */
}
